@@ -16,25 +16,31 @@
  *  License along with this library; if not, write to the Free Software              *
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA       *
  *************************************************************************************/
-
 #include "xrandroutput.h"
-#include "xrandrmode.h"
-#include "xrandrconfig.h"
-#include "xrandr.h"
-#include "output.h"
+
 #include "config.h"
+#include "xrandr.h"
+#include "xrandrconfig.h"
+#include "xrandrmode.h"
 #include "../utils.h"
 
-#include <QRect>
+#include <xcb/render.h>
 
 Q_DECLARE_METATYPE(QList<int>)
 
+#define DOUBLE_TO_FIXED(d) ((xcb_render_fixed_t) ((d) * 65536))
+#define FIXED_TO_DOUBLE(f) ((double) ((f) / 65536.0))
+
+xcb_render_fixed_t fOne = DOUBLE_TO_FIXED(1);
+xcb_render_fixed_t fZero = DOUBLE_TO_FIXED(0);
+
 XRandROutput::XRandROutput(xcb_randr_output_t id, XRandRConfig *config)
     : QObject(config)
     , m_config(config)
     , m_id(id)
-    , m_type(KScreen::Output::Unknown)
     , m_primary(false)
+    , m_type(KScreen::Output::Unknown)
+    , m_replicationSource(XCB_NONE)
     , m_crtc(nullptr)
 {
     init();
@@ -89,6 +95,7 @@ XRandRMode* XRandROutput::currentMode() const
     if (!m_crtc) {
         return nullptr;
     }
+
     unsigned int modeId = m_crtc->mode();
     if (!m_modes.contains(modeId)) {
         return nullptr;
@@ -99,7 +106,14 @@ XRandRMode* XRandROutput::currentMode() const
 
 KScreen::Output::Rotation XRandROutput::rotation() const
 {
-    return static_cast<KScreen::Output::Rotation>(m_crtc ? m_crtc->rotation() : XCB_RANDR_ROTATION_ROTATE_0);
+    return static_cast<KScreen::Output::Rotation>(m_crtc ? m_crtc->rotation() :
+                                                           XCB_RANDR_ROTATION_ROTATE_0);
+}
+
+bool XRandROutput::isHorizontal() const
+{
+    const auto rot = rotation();
+    return rot == KScreen::Output::Rotation::None || rot == KScreen::Output::Rotation::Inverted;
 }
 
 QByteArray XRandROutput::edid() const
@@ -115,20 +129,26 @@ XRandRCrtc* XRandROutput::crtc() const
     return m_crtc;
 }
 
+xcb_randr_output_t XRandROutput::replicationSource() const
+{
+    return m_replicationSource;
+}
+
 void XRandROutput::update()
 {
     init();
 }
 
-void XRandROutput::update(xcb_randr_crtc_t crtc, xcb_randr_mode_t mode, xcb_randr_connection_t conn, bool primary)
+void XRandROutput::update(xcb_randr_crtc_t crtc, xcb_randr_mode_t mode, xcb_randr_connection_t conn,
+                          bool primary)
 {
-    qCDebug(KSCREEN_XRANDR) << "XRandROutput" << m_id << "update";
-    qCDebug(KSCREEN_XRANDR) << "\tm_connected:" << m_connected;
-    qCDebug(KSCREEN_XRANDR) << "\tm_crtc" << m_crtc;
-    qCDebug(KSCREEN_XRANDR) << "\tCRTC:" << crtc;
-    qCDebug(KSCREEN_XRANDR) << "\tMODE:" << mode;
-    qCDebug(KSCREEN_XRANDR) << "\tConnection:" << conn;
-    qCDebug(KSCREEN_XRANDR) << "\tPrimary:" << primary;
+    qCDebug(KSCREEN_XRANDR) << "XRandROutput" << m_id << "update" << "\n"
+                            << "\tm_connected:" << m_connected << "\n"
+                            << "\tm_crtc" << m_crtc << "\n"
+                            << "\tCRTC:" << crtc << "\n"
+                            << "\tMODE:" << mode << "\n"
+                            << "\tConnection:" << conn << "\n"
+                            << "\tPrimary:" << primary;
 
     // Connected or disconnected
     if (isConnected() != (conn == XCB_RANDR_CONNECTION_CONNECTED)) {
@@ -194,17 +214,21 @@ void XRandROutput::init()
 
     XCB::PrimaryOutput primary(XRandR::rootWindow());
 
-    m_name = QString::fromUtf8((const char *) xcb_randr_get_output_info_name(outputInfo.data()), outputInfo->name_len);
+    m_name = QString::fromUtf8((const char *) xcb_randr_get_output_info_name(outputInfo.data()),
+                               outputInfo->name_len);
     m_type = fetchOutputType(m_id, m_name);
     m_icon = QString();
     m_connected = (xcb_randr_connection_t) outputInfo->connection;
     m_primary = (primary->output == m_id);
+
     xcb_randr_output_t *clones = xcb_randr_get_output_info_clones(outputInfo.data());
     for (int i = 0; i < outputInfo->num_clones; ++i) {
         m_clones.append(clones[i]);
     }
+
     m_widthMm = outputInfo->mm_width;
     m_heightMm = outputInfo->mm_height;
+
     m_crtc = m_config->crtc(outputInfo->crtc);
     if (m_crtc) {
         m_crtc->connectOutput(m_id);
@@ -217,7 +241,9 @@ void XRandROutput::init()
 void XRandROutput::updateModes(const XCB::OutputInfo &outputInfo)
 {
     /* Init modes */
-    XCB::ScopedPointer<xcb_randr_get_screen_resources_reply_t> screenResources(XRandR::screenResources());
+    XCB::ScopedPointer<xcb_randr_get_screen_resources_reply_t>
+            screenResources(XRandR::screenResources());
+
     Q_ASSERT(screenResources);
     if (!screenResources) {
         return;
@@ -247,7 +273,8 @@ void XRandROutput::updateModes(const XCB::OutputInfo &outputInfo)
     }
 }
 
-KScreen::Output::Type XRandROutput::fetchOutputType(xcb_randr_output_t outputId, const QString &name)
+KScreen::Output::Type XRandROutput::fetchOutputType(xcb_randr_output_t outputId,
+                                                    const QString &name)
 {
     QString type = QString::fromUtf8(typeFromProperty(outputId));
     if (type.isEmpty()) {
@@ -266,11 +293,10 @@ QByteArray XRandROutput::typeFromProperty(xcb_randr_output_t outputId)
         return type;
     }
 
-    char *connectorType;
-
     auto cookie = xcb_randr_get_output_property(XCB::connection(), outputId, atomType->atom,
                                                 XCB_ATOM_ANY, 0, 100, false, false);
-    XCB::ScopedPointer<xcb_randr_get_output_property_reply_t> reply(xcb_randr_get_output_property_reply(XCB::connection(), cookie, nullptr));
+    XCB::ScopedPointer<xcb_randr_get_output_property_reply_t>
+            reply(xcb_randr_get_output_property_reply(XCB::connection(), cookie, nullptr));
     if (!reply) {
         return type;
     }
@@ -285,7 +311,7 @@ QByteArray XRandROutput::typeFromProperty(xcb_randr_output_t outputId)
         return type;
     }
 
-    connectorType = xcb_get_atom_name_name(atomName);
+    char *connectorType = xcb_get_atom_name_name(atomName);
     if (!connectorType) {
         return type;
     }
@@ -294,6 +320,158 @@ QByteArray XRandROutput::typeFromProperty(xcb_randr_output_t outputId)
     return type;
 }
 
+bool isScaling(const xcb_render_transform_t &tr)
+{
+    return tr.matrix11 != fZero && tr.matrix12 == fZero && tr.matrix13 == fZero &&
+           tr.matrix21 == fZero && tr.matrix22 != fZero && tr.matrix23 == fZero &&
+           tr.matrix31 == fZero && tr.matrix32 == fZero && tr.matrix33 == fOne;
+}
+
+xcb_render_transform_t zeroMatrix()
+{
+    return { DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+             DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+             DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0) };
+}
+
+xcb_render_transform_t XRandROutput::currentTransform() const
+{
+    auto cookie = xcb_randr_get_crtc_transform(XCB::connection(), m_crtc->crtc());
+    xcb_generic_error_t *error = nullptr;
+    auto *reply = xcb_randr_get_crtc_transform_reply(XCB::connection(), cookie, &error);
+    if (error) {
+        return zeroMatrix();
+    }
+
+    const xcb_render_transform_t transform = reply->pending_transform;
+    free(reply);
+    return transform;
+}
+
+QSizeF XRandROutput::scaledSize(xcb_render_transform_t transform) const
+{
+    const QSize ownSize = size();
+    if (!ownSize.isValid()) {
+        return QSize();
+    }
+
+    const qreal width = FIXED_TO_DOUBLE(transform.matrix11) * ownSize.width();
+    const qreal height = FIXED_TO_DOUBLE(transform.matrix22) * ownSize.height();
+
+    return QSizeF(width, height);
+}
+
+bool XRandROutput::isReplicaOf(XRandROutput *output, xcb_render_transform_t ownTransform) const
+{
+    if (output->id() == m_id) {
+        return false;
+    }
+    if (output->position() != position()) {
+        return false;
+    }
+    if (output->replicationSource() != XCB_NONE) {
+        return false;
+    }
+
+    const QSizeF sSize = scaledSize(ownTransform);
+    if (!sSize.isValid()) {
+        return false;
+    }
+
+    const auto outputTransform = output->currentTransform();
+    if (!isScaling(outputTransform)) {
+        return false;
+    }
+
+    if (sSize != output->scaledSize(outputTransform)) {
+        return false;
+    }
+
+    return true;
+}
+
+xcb_render_transform_t unityTransform()
+{
+    return { DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0),
+             DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1), DOUBLE_TO_FIXED(0),
+             DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(0), DOUBLE_TO_FIXED(1) };
+}
+
+xcb_render_transform_t XRandROutput::getReplicationTransform(XRandROutput *source)
+{
+    if (!source) {
+        return unityTransform();
+    }
+
+    const auto *sourceMode = source->currentMode();
+    const auto *ownMode = currentMode();
+    if (!sourceMode || !ownMode) {
+        return unityTransform();
+    }
+
+    QSize sourceSize = sourceMode->size();
+    QSize size = ownMode->size();
+
+    if (isHorizontal()) {
+        if (!source->isHorizontal()) {
+            sourceSize.transpose();
+        }
+    } else if (source->isHorizontal()) {
+        size.transpose();
+    }
+
+    const qreal widthFactor = sourceSize.width() / (qreal)size.width();
+    const qreal heightFactor = sourceSize.height() / (qreal)size.height();
+
+    xcb_render_transform_t transform = unityTransform();
+    transform.matrix11 = DOUBLE_TO_FIXED(widthFactor);
+    transform.matrix22 = DOUBLE_TO_FIXED(heightFactor);
+
+    return transform;
+}
+
+bool XRandROutput::updateReplication()
+{
+    XRandROutput *source = m_config->output(m_replicationSource);
+    if (source && (!source->isEnabled() || !source->isConnected())) {
+        return false;
+    }
+
+    xcb_render_transform_t transform = getReplicationTransform(source);
+    QByteArray filterName(isScaling(transform) ? "bilinear" : "nearest");
+
+    auto cookie = xcb_randr_set_crtc_transform_checked(XCB::connection(),
+                                                       m_crtc->crtc(),
+                                                       transform,
+                                                       filterName.size(), filterName.data(),
+                                                       0, nullptr);
+    xcb_generic_error_t *error = xcb_request_check(XCB::connection(), cookie);
+    if (error) {
+        qCDebug(KSCREEN_XRANDR) << "Error on replication transformation!";
+        free(error);
+        return false;
+    }
+    free(error);
+    return true;
+}
+
+bool XRandROutput::setReplicationSource(xcb_randr_output_t source)
+{
+    if (!m_crtc) {
+        return false;
+    }
+    if (m_replicationSource == source) {
+        return true;
+    }
+    xcb_randr_output_t oldSource = m_replicationSource;
+    m_replicationSource = source;
+    if (!updateReplication()) {
+        m_replicationSource = oldSource;
+        return false;
+    }
+    return true;
+}
+
 KScreen::OutputPtr XRandROutput::toKScreenOutput() const
 {
     KScreen::OutputPtr kscreenOutput(new KScreen::Output);
@@ -335,9 +513,9 @@ KScreen::OutputPtr XRandROutput::toKScreenOutput() const
             kscreenOutput->setRotation(rotation());
             kscreenOutput->setCurrentModeId(currentModeId());
         }
+        kscreenOutput->setReplicationSource(m_replicationSource);
     }
 
-
     kscreenOutput->blockSignals(signalsBlocked);
     return kscreenOutput;
 }
