@@ -56,6 +56,8 @@ class KSCREEN_EXPORT Output : public QObject
         Q_PROPERTY(bool enabled READ isEnabled WRITE setEnabled NOTIFY isEnabledChanged)
         Q_PROPERTY(bool primary READ isPrimary WRITE setPrimary NOTIFY isPrimaryChanged)
         Q_PROPERTY(QList<int> clones READ clones WRITE setClones NOTIFY clonesChanged)
+        Q_PROPERTY(int replicationSource READ replicationSource
+                   WRITE setReplicationSource NOTIFY replicationSourceChanged)
         Q_PROPERTY(KScreen::Edid* edid READ edid CONSTANT)
         Q_PROPERTY(QSize sizeMm READ sizeMm CONSTANT)
         Q_PROPERTY(qreal scale READ scale WRITE setScale NOTIFY scaleChanged)
@@ -106,9 +108,22 @@ class KSCREEN_EXPORT Output : public QObject
          *
          * @return identifying hash of this output
          * @since 5.15
+         * @deprecated
+         * @see hashMd5
          */
         QString hash() const;
 
+        /**
+         * Returns an identifying hex encoded MD5-hash for this output.
+         *
+         * The hash is calculated either via the edid hash or if no
+         * edid is available by the output name, which is hashed as well.
+         *
+         * @return identifying hash of this output
+         * @since 5.17
+         */
+        QString hashMd5() const;
+
         Type type() const;
         void setType(Type type);
 
@@ -158,6 +173,14 @@ class KSCREEN_EXPORT Output : public QObject
         QSize size() const;
         void setSize(const QSize& size);
 
+        /**
+         * Returns either current mode size or if not available preferred one or if also not
+         * available the first one in the ModeList.
+         *
+         * @return mode size
+         */
+        QSize enforcedModeSize() const;
+
         Rotation rotation() const;
         void setRotation(Rotation rotation);
         /**
@@ -177,9 +200,41 @@ class KSCREEN_EXPORT Output : public QObject
         bool isPrimary() const;
         void setPrimary(bool primary);
 
+        /**
+         * @brief Immutable clones because of hardware restrictions
+         *
+         * Clones are set symmetcally on all outputs. The list contains ids
+         * for all other outputs being clones of this output.
+         *
+         * @return List of output ids being clones of each other.
+         */
         QList<int> clones() const;
+        /**
+         * @brief Set the clones list.
+         *
+         * When this output is part of a configuration this call is followed by
+         * similar calls on other outputs making the lists in all outputs
+         * symmetric.
+         * @param outputlist
+         */
         void setClones(const QList<int> &outputlist);
 
+        /**
+         * @brief Provides the source for an ongoing replication
+         *
+         * If the returned output id is non-null this output is a replica of the
+         * returned output. If null is returned the output is no replica of any
+         * other output.
+         *
+         * @return Replication source output id of this output
+         */
+        int replicationSource() const;
+        /**
+         * @brief Set the replication source.
+         * @param source
+         */
+        void setReplicationSource(int source);
+
         void setEdid(const QByteArray &rawData);
 
         /**
@@ -199,6 +254,16 @@ class KSCREEN_EXPORT Output : public QObject
         QSize sizeMm() const;
         void setSizeMm(const QSize &size);
 
+        /**
+         * Returns if the output needs to be taken account for in the overall compositor/screen
+         * space and if it should be depicted on its own in a graphical view for repositioning.
+         *
+         * @return true if the output is positionable in compositor/screen space.
+         *
+         * @since 5.17
+         */
+        bool isPositionable() const;
+
         /**
          * Returns a rectangle containing the currently set output position and
          * size.
@@ -256,6 +321,7 @@ class KSCREEN_EXPORT Output : public QObject
         void isEnabledChanged();
         void isPrimaryChanged();
         void clonesChanged();
+        void replicationSourceChanged();
         void scaleChanged();
         void followPreferredModeChanged(bool followPreferredMode);
 
